local t = TRCLOOT
local LSM = LibStub("LibSharedMedia-3.0")

t.fromPawnStatName = {
    ["Versatility"] = "Versatility",
    ["MasteryRating"] = "Mastery",
    ["CritRating"] = "Crit",
    ["HasteRating"] = "Haste",
    ["Intellect"] = "Intellect",
    ["Strength"] = "Strength",
    ["Agility"] = "Agility",
}


local resetPosition = function() 
    local t = TRCLOOT

    if t.svc and t.svc.anchorOffsetX then
        t.drawFrame:ClearAllPoints()
        t.svc.anchorOffsetX = 0
        t.svc.anchorOffsetY = 0
        t.svc.anchorSelf = "CENTER"
        t.svc.anchorOther = "CENTER"
        t.drawFrame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
        t.drawFrame:Show()
    end
end

local addTooltip = function(frame, text, anchor, anchorTo)
    if text then
        frame:SetScript("OnEnter", function()
            GameTooltip:SetOwner(frame)
            GameTooltip:SetPoint(anchor, frame, anchorTo)
            GameTooltip:AddLine(text,1,.82,0, true)
            GameTooltip:Show()
        end)

        frame:SetScript("OnLeave", function()
            GameTooltip:Hide()
        end)
    end

end

local addCheckBox = function(varName, text, description, width, parentFrame, anchorTo, anchorAt, xOffset, yOffset)

    local checkBox = CreateFrame("CheckButton", t.ADDON_NAME..varName.."CheckBox", parentFrame, "UICheckButtonTemplate")
    checkBox:SetPoint("TOPLEFT", anchorTo, anchorAt, xOffset, yOffset)
    checkBox:SetWidth(30)
    checkBox:SetHeight(30)
    checkBox.tooltipText = description


    checkBox:SetChecked(t.getVar(varName))


    addTooltip(checkBox, description, "BOTTOM", "TOP")

    local txt = _G[checkBox:GetName().."Text"]
    txt:SetText(text)

    checkBox:SetScript("OnClick", function(self, button)
        t.svc[varName] = self:GetChecked()
        t.drawReports()
    end)

    checkBox:Show()

    return checkBox
end

local addButton = function(text, description, frame, anchorTo, onClickF)
    local button = CreateFrame("Button", t.ADDON_NAME.."ResetPositionButton", frame) 
    button:SetPoint("TOPLEFT", anchorTo, "BOTTOMLEFT", 0, 0)
    button:SetWidth(180)
    button:SetHeight(36)

    button:SetNormalFontObject("GameFontNormalSmall")

    button:SetNormalTexture("Interface/Buttons/UI-Panel-Button-Up")
    button:SetHighlightTexture("Interface/Buttons/UI-Panel-Button-Highlight")
    button:SetPushedTexture("Interface/Buttons/UI-Panel-Button-Down")

    button:SetDisabledFontObject(GameFontDisable)
    button:SetHighlightFontObject(GameFontHighlight)
    button:SetNormalFontObject(GameFontNormal)

    local fo = button:CreateFontString()
    --fo:SetFont("Fonts/FRIZQT__.TTF",12)
    fo:SetFontObject("GameFontNormalSmall")
    fo:SetPoint("TOPLEFT", button, "TOPLEFT", 16,-7)
    fo:SetText(text)
    button:SetFontString(fo)

    button:SetScript("OnClick", onClickF)


    addTooltip(button, description, "LEFT", "RIGHT")


    return button
end









local addDropdown = function(name, text, ttText, onInit, selectedId, width, frame, anchorTo, anchorAt, yOffset)
    -- Spec text
    local textObject = frame:CreateFontString(t.ADDON_NAME..name.."Title", "ARTWORK", "GameFontNormal")
    textObject:SetPoint("TOPLEFT", anchorTo, anchorAt, 0,yOffset)
    textObject:SetText(text)
    textObject:SetJustifyH("LEFT")
    textObject:Show()


    -- Spec dropdown
    local dropdown = CreateFrame("Button", t.ADDON_NAME..name, frame, "UIDropDownMenuTemplate")

    dropdown:ClearAllPoints()
    dropdown:SetPoint("TOPLEFT", textObject, "BOTTOMLEFT", -20, -2)
    dropdown:Show()


    if ttText then
        addTooltip(dropdown, ttText, "BOTTOMRIGHT", "BOTTOMLEFT")
    end



    UIDropDownMenu_Initialize(dropdown, onInit)
    UIDropDownMenu_SetWidth(dropdown, width-24);
    UIDropDownMenu_SetButtonWidth(dropdown, width)
    if selectedId > 0 then
        UIDropDownMenu_SetSelectedID(dropdown, selectedId)
    end
    UIDropDownMenu_JustifyText(dropdown, "LEFT")

    return textObject, dropdown

end







local generalFrame = CreateFrame("Frame", t.ADDON_NAME.."ConfigFrame", InterfaceOptionsFramePanelContainer)
generalFrame.name = t.ADDON_NAME
generalFrame:Hide()
generalFrame:SetScript("OnShow", function(frame)
    
    t.currentEditingSpec = t.currentEditingSpec or GetSpecialization()

    local title = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
    title:SetPoint("TOPLEFT", 16, -16)
    title:SetText("Raven Loot")

    local subtitle = frame:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
    subtitle:SetHeight(35)
    subtitle:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -8)
    subtitle:SetPoint("RIGHT", frame, -32, 0)
    subtitle:SetNonSpaceWrap(true)
    subtitle:SetJustifyH("LEFT")
    subtitle:SetJustifyV("TOP")
    subtitle:SetText("General settings.")

    local uiScale = GetCVar("uiScale")

    --[[ LEFT SIDE ]]--
    local leftFrame = CreateFrame("Frame", t.ADDON_NAME.."ConfigFrameLeft")
    leftFrame:SetWidth(frame:GetWidth() / 2 * uiScale)
    leftFrame:SetHeight(frame:GetHeight() * uiScale)
    leftFrame:SetPoint("TOPLEFT", subtitle, "BOTTOMLEFT", 0,0)
    leftFrame:Show()


    local lastBox
    lastBox = addCheckBox("showRealmNames", "Show Realm Names", "Show realm names after player names.", 300, frame, leftFrame, "TOPLEFT")
    lastBox = addCheckBox("showMoveMe", "Unlock Anchor", "Show the 'Move Me' block to move around the window.", 300, frame, lastBox, "BOTTOMLEFT")
    --lastBox = addCheckBox("onlyShowEquipable", "Only show equipable loot", "Only show loot that you can equip yourself.", 300, frame, lastBox, "BOTTOMLEFT")
    --lastBox = addCheckBox("compareNonTradable", "Automatically compare non-tradable loot", "Automatically compares (i.e. offer, but only for you) loot that drops if it is not tradable.", 300, frame, lastBox, "BOTTOMLEFT")
    lastBox = addButton("Reset Position", "Resets the position of the loot window to the center of the screen. Useful in case you moved it off the screen.", frame, lastBox, resetPosition)


    local debugText = frame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
    debugText:SetPoint("TOPLEFT", lastBox, "BOTTOMLEFT", 0,-10)

    --[[
    debugText:SetText("Debug")
    debugText:SetJustifyH("LEFT")
    debugText:Show()

    lastBox = addCheckBox("reportAllEquipable", "Report every item as equipable", "Should not be enabled in practise. If enabled, you will respond to all offered items, even if you cannot equip them.", 300, frame, debugText, "BOTTOMLEFT")
    ]]--


    --[[ RIGHT SIDE ]]--

    local rightFrame = CreateFrame("Frame", t.ADDON_NAME.."ConfigFrameRight")
    rightFrame:SetWidth(frame:GetWidth() / 2 * uiScale)
    rightFrame:SetHeight(frame:GetHeight() * uiScale)
    rightFrame:SetPoint("TOPLEFT", leftFrame, "TOPRIGHT", 0,0)
    rightFrame:Show()

    -- Font text
    local fontText = frame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
    fontText:SetPoint("TOPLEFT", rightFrame, "TOPLEFT", 0,0)
    fontText:SetText("Font")
    fontText:SetJustifyH("LEFT")
    fontText:Show()


    -- Font dropdown
    local fontDropdown = CreateFrame("Button", t.ADDON_NAME.."FontDropdownMenu", frame, "UIDropDownMenuTemplate")

    fontDropdown:ClearAllPoints()
    fontDropdown:SetPoint("TOPLEFT", fontText, "BOTTOMLEFT", -20, -2)
    fontDropdown:Show()


    lastBox = addCheckBox("swapAnchor", "Show scores on the left", "Swaps some of the anchor points around, showing the scores on the left side of the offered loot, making the layout more suitable for placing it on the right side of the screen.", 300, frame, fontDropdown, "BOTTOMLEFT", 20, 0)
    lastBox = addCheckBox("growUpwards", "Grow upwards", "Offered items grow upwards instead of downwards, making the layout more suitable for placing it on the bottom of the screen.", 300, frame, lastBox, "BOTTOMLEFT", 0, 0)





    local function initialize(self, level)

        local info = UIDropDownMenu_CreateInfo()
        for k,v in pairs(LSM:List("font")) do
            info = UIDropDownMenu_CreateInfo()
            info.text = v
            info.value = v
            info.func = function(self)
                UIDropDownMenu_SetSelectedID(fontDropdown, self:GetID())
                if t.svc then
                    t.svc.font = v
                    t.svc.fontPath = LSM:Fetch("font", v)
                    t.drawReports()
                end
            end
            UIDropDownMenu_AddButton(info, level)
        end
    end


    local currentFontIndex = 1
    if t.svc and t.svc.font then
        for k, v in pairs(LSM:List("font")) do
            if v == t.svc.font then
                currentFontIndex = k
            end
        end
    end


    UIDropDownMenu_Initialize(fontDropdown, initialize)
    UIDropDownMenu_SetWidth(fontDropdown, 240);
    UIDropDownMenu_SetButtonWidth(fontDropdown, 264)
    UIDropDownMenu_SetSelectedID(fontDropdown, currentFontIndex)
    UIDropDownMenu_JustifyText(fontDropdown, "LEFT")

    local Refresh;
    function Refresh()
        if not frame:IsVisible() then return end
    end

    frame:SetScript("OnShow", Refresh) 
    Refresh()
end)

InterfaceOptions_AddCategory(generalFrame)


local drawStatGroup = function(titleString, group, boxes, startIndex, parentFrame, anchorTo, anchorAt)
    t.currentEditingSpec = t.currentEditingSpec or GetSpecialization()
    local specInfo = t.getVar("specs")
    local currentSpecInfo = specInfo[t.currentEditingSpec]

    local counter = startIndex

    -- Title
    local title = parentFrame:CreateFontString(t.ADDON_NAME..titleString.."Title", "ARTWORK", "GameFontNormal")
    title:SetPoint("TOPLEFT", anchorTo, anchorAt, 0,-10)
    title:SetText(titleString)
    title:SetJustifyH("LEFT")
    title:Show()

    anchorTo = title

    for fullName, shortName in pairs(group) do

        boxes[counter] = {}

        -- Contains all ui elements from this stat
        local s = boxes[counter]
        s.fullName = fullName
        s.shortName = shortName

        s.statBox = parentFrame:CreateTexture(nil, "BACKGROUND", nil, -1)
        s.statBox:SetPoint("TOPLEFT", anchorTo, "BOTTOMLEFT", 0, -5)
        s.statBox:SetHeight(20)
        s.statBox:SetWidth(200)

        anchorTo = s.statBox

        s.statText = parentFrame:CreateFontString(nil, "FULLSCREEN")
        s.statText:SetFont("Fonts\\FRIZQT__.TTF", 11, "OUTLINE")
        s.statText:SetPoint("LEFT", s.statBox, "LEFT", 5 , 0)
        s.statText:SetText(shortName)
        s.statText:Show()

        local buttonName = t.ADDON_NAME.."Stat"..shortName.."ConfirmButton"
        s.confirmButton = CreateFrame("Button", buttonName, parentFrame, "UIPanelButtonTemplate")
        s.confirmButton:SetWidth(30)
        s.confirmButton:SetHeight(s.statBox:GetHeight())
        s.confirmButton:SetPoint("RIGHT", s.statBox, "RIGHT", 0, 0)
        s.confirmButton.text = _G[buttonName.."Text"]
        s.confirmButton.text:SetText("OK")
        s.confirmButton:Hide()

        s.editBox = CreateFrame("EditBox", t.ADDON_NAME.."Stat"..shortName.."EditBox", parentFrame, "InputBoxTemplate")
        s.editBox:SetPoint("RIGHT", s.confirmButton, "LEFT", -5, 0)
        --local statWeightValue = string.format("%.2f", t.svc.statWeights[shortName])
        local statWeightValue = string.format("%.2f", (currentSpecInfo.sw[shortName] or 0))
        s.editBox:SetText(statWeightValue)
        s.editBox:SetFont("Fonts\\FRIZQT__.TTF", 11, "OUTLINE")
        s.editBox:SetFrameStrata("FULLSCREEN")
        s.editBox:SetWidth(44)
        s.editBox:SetHeight(s.statBox:GetHeight())
        s.editBox:SetAutoFocus(false)
        s.editBox.counterVal = counter 
        s.editBox:Show()


        -- On focus gain
        s.editBox:SetScript("OnEditFocusGained", function(self)
            local s = boxes[self.counterVal] 
            s.confirmButton:Show()
            s.editBox:HighlightText()
        end)


        -- On focus lost
        s.editBox:SetScript("OnEditFocusLost", function(self)

            local currentSpecInfo = t.getVar("specs")[t.currentEditingSpec]

            local s = boxes[self.counterVal] 
            s.confirmButton:Hide()
            s.editBox:HighlightText(0,0)
            --t.svc.defaultWeights = false
            --t.svc.statWeights[shortName] = s.editBox:GetNumber()
            currentSpecInfo.sw[shortName] = s.editBox:GetNumber()

            s.editBox:SetText(string.format("%.2f", currentSpecInfo.sw[shortName]))
        end)







        counter = counter+1
    end

    return counter, anchorTo

end




-- Statweight panel
local statweightFrame = CreateFrame("Frame", t.ADDON_NAME.."StatweightFrame", InterfaceOptionsFramePanelContainer)
statweightFrame.name = "Specializations"
statweightFrame:Hide()
statweightFrame.parent = generalFrame
statweightFrame:SetScript("OnShow", function(frame)

    t.currentEditingSpec = t.currentEditingSpec or GetSpecialization()

    local specInfo = t.getVar("specs")
    local currentSpecInfo = specInfo[t.currentEditingSpec]

    -- Title
    local title = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
    title:SetPoint("TOPLEFT", 16, -16)
    title:SetText("Specializations")

    -- Subtitle
    local subtitle = frame:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
    subtitle:SetHeight(50)
    subtitle:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -8)
    subtitle:SetPoint("RIGHT", frame, -32, 0)
    subtitle:SetNonSpaceWrap(true)
    subtitle:SetJustifyH("LEFT")
    subtitle:SetJustifyV("TOP")
    subtitle:SetText("Here you can set your stat-weights. These stat-weights will be used to calculate a score for every item that drops. This way, for every item that drops it can be determined for who it is the largest upgrade. All stat-weights will be normalized, so only their relative value matters, not their absolute value.")

    local uiScale = GetCVar("uiScale")

    --[[ LEFT SIDE ]]--
    local leftFrame = CreateFrame("Frame", t.ADDON_NAME.."StatConfigFrameLeft")
    leftFrame:SetWidth(frame:GetWidth() / 2 * uiScale)
    leftFrame:SetHeight(frame:GetHeight() * uiScale)
    leftFrame:SetPoint("TOPLEFT", subtitle, "BOTTOMLEFT", 0,0)
    leftFrame:Show()








    -- Spec dropdown
    local specs = {}
    for i=1, GetNumSpecializations() do
        local name = select(2, GetSpecializationInfo(i))
        table.insert(specs, name)
    end

    local name = "SpecDropdown"

    local function initSpecs(self, level)

        local info = UIDropDownMenu_CreateInfo()
        for k,v in pairs(specs) do
            info = UIDropDownMenu_CreateInfo()
            info.text = v
            info.value = v
            info.func = function(self)
                UIDropDownMenu_SetSelectedID(_G[t.ADDON_NAME..name], self:GetID())

                --Save and unfocus sw boxes
                if t.options and t.options.swBoxes then
                    local boxes = t.options.swBoxes

                    for n=1, #boxes do
                        boxes[n].editBox:ClearFocus()
                    end
                end

                -- Update spec
                t.currentEditingSpec = k


                -- Trigger refresh
                frame:Hide()
                frame:Show()
            end
            UIDropDownMenu_AddButton(info, level)
        end
    end

    local last = addDropdown(name, "Specialization", nil, initSpecs, t.currentEditingSpec, 200, frame, leftFrame, "TOPLEFT")

    -- Enable spec checkboxx
    local enableCheckbox = CreateFrame("CheckButton", t.ADDON_NAME.."EnableSpecCheckBox", frame, "UICheckButtonTemplate")
    enableCheckbox:SetPoint("TOPLEFT", last, "BOTTOMLEFT", 0, -40)
    enableCheckbox:SetWidth(30)
    enableCheckbox:SetHeight(30)
    enableCheckbox.tooltipText = "Hi"

    enableCheckbox:SetChecked(currentSpecInfo["enable"])

    local txt = _G[enableCheckbox:GetName().."Text"]
    txt:SetText("Enable spec")

    addTooltip(enableCheckbox, "Respond to loot offers with this specialization.", "BOTTOM", "TOP")

    enableCheckbox:SetScript("OnClick", function(self, button)
        currentSpecInfo["enable"] = self:GetChecked()
        -- Trigger refresh
        frame:Hide()
        frame:Show()
    end)

    enableCheckbox:Show()
    last  = enableCheckbox


    -- Main spec checkbox
    local msCheckbox = CreateFrame("CheckButton", t.ADDON_NAME.."MsCheckBox", frame, "UICheckButtonTemplate")
    msCheckbox:SetPoint("TOPLEFT", last, "BOTTOMLEFT", 0, 0)
    msCheckbox:SetWidth(30)
    msCheckbox:SetHeight(30)
    msCheckbox.tooltipText = "Bye"

    msCheckbox:SetChecked(currentSpecInfo["ms"])

    local txt = _G[msCheckbox:GetName().."Text"]
    txt:SetText("Report as mainspec")
    addTooltip(msCheckbox, "Lists this specialization in the Mainspec list.", "BOTTOM", "TOP")

    msCheckbox:SetScript("OnClick", function(self, button)
        currentSpecInfo["ms"] = self:GetChecked()
    end)

    msCheckbox:Show()
    last = msCheckbox



    -- Equipment dropdown

    local name = "EquipmentSetDropdown"

    local function initEquipmentSets(self, level)

        local equipmentSets = {}
        for i=1, C_EquipmentSet.GetNumEquipmentSets() do
            local name = C_EquipmentSet.GetEquipmentSetInfo(i-1)
            table.insert(equipmentSets, name)
        end

        local info = UIDropDownMenu_CreateInfo()

        -- Add empty set
        info = UIDropDownMenu_CreateInfo()
        info.text = "[Use spec assigned set]"
        info.value = -1
        info.func = function(self)
            UIDropDownMenu_SetSelectedID(_G[t.ADDON_NAME..name], self:GetID())
            currentSpecInfo["equipmentSet"] = -1
        end
        UIDropDownMenu_AddButton(info, level)

        -- Add other sets
        for k,v in pairs(equipmentSets) do
            info = UIDropDownMenu_CreateInfo()
            info.text = v
            info.value = k-1
            info.func = function(self)
                UIDropDownMenu_SetSelectedID(_G[t.ADDON_NAME..name], self:GetID())
                currentSpecInfo["equipmentSet"] = k-1
            end
            UIDropDownMenu_AddButton(info, level)
        end
    end

    local selectedId = nil
    --[[
    for i=1, GetNumEquipmentSets() do
        if C_EquipmentSet.GetEquipmentSetInfo(i) == currentSpecInfo["equipmentSet"] then
            selectedId = i+1
        end
    end
    if not selectedId then
        selectedId = 1
    end
    ]]
    local set = currentSpecInfo["equipmentSet"]
    local specName = select(2, GetSpecializationInfo(t.currentEditingSpec))

    if set == -1 then selectedId = 1 else selectedId = set+2 end

    local ttText = "If not currently in the selected specialization, items from this equipment set will be used to compare dropped loot with instead of equiped gear. All the items in the equipment set must be in your inventory (not bank) for this to work.\n\n[Use spec assigned set] will take the set that you have marked for this spec in the Blizzard Equipment Manager."
    last = addDropdown(name, "Equipment Set", ttText, initEquipmentSets, selectedId, 200, frame, last, "BOTTOMLEFT", -10)



    -- Gem stat dropdown


    local name = "GemStatDropdown"

    local function initGemStat(self, level)

        local info = UIDropDownMenu_CreateInfo()

        info.text = ""
        info.value = nil
        info.func = function(self)
            UIDropDownMenu_SetSelectedID(_G[t.ADDON_NAME..name], self:GetID())
            currentSpecInfo["gemStat"] = fullName
        end

        UIDropDownMenu_AddButton(info, level)

        for shortName, fullName in pairs(t.reverseStm) do
            info = UIDropDownMenu_CreateInfo()
            info.text = shortName
            info.value = fullName
            info.func = function(self)
                UIDropDownMenu_SetSelectedID(_G[t.ADDON_NAME..name], self:GetID())
                currentSpecInfo["gemStat"] = fullName
            end
            UIDropDownMenu_AddButton(info, level)
        end
    end


    local currentGemStat = currentSpecInfo["gemStat"]
    local currentIndex = 1
    local i = 1
    for shortName, fullName in pairs(t.reverseStm) do
        i = i + 1
        if fullName == currentGemStat then
            currentIndex = i
        end
    end



    local ttText = "Select the stat and amount to fill sockets with. For example, if you would put in a Deadly Eye of Prophecy in a socket, enter 'Critical Strike' and '150'."
    local lastText, lastDropdown = addDropdown(name, "Gem stats", ttText, initGemStat, currentIndex, 138, frame, last, "BOTTOMLEFT", -40)




    -- Gem stat value
    local gemStatValEditBox = CreateFrame("EditBox", t.ADDON_NAME.."GemStatValEditBox", frame, "InputBoxTemplate")
    gemStatValEditBox:SetPoint("LEFT", lastDropdown, "RIGHT", 0, 0)
    --local statWeightValue = string.format("%.2f", t.svc.statWeights[shortName])
    local gemStatVal = currentSpecInfo["gemStatVal"] or 0
    gemStatValEditBox:SetText(string.format("%d", gemStatVal))
    gemStatValEditBox:SetFont("Fonts\\FRIZQT__.TTF", 11, "OUTLINE")
    gemStatValEditBox:SetFrameStrata("FULLSCREEN")
    gemStatValEditBox:SetWidth(44)
    gemStatValEditBox:SetHeight(20)
    gemStatValEditBox:SetAutoFocus(false)
    gemStatValEditBox:Show()

    local buttonName = t.ADDON_NAME.."GemStatValConfirmButton"
    local gemStatValConfirmButton = CreateFrame("Button", buttonName, frame, "UIPanelButtonTemplate")
    gemStatValConfirmButton:SetWidth(30)
    gemStatValConfirmButton:SetHeight(24)
    gemStatValConfirmButton:SetPoint("LEFT", gemStatValEditBox, "RIGHT", 0, 0)
    gemStatValConfirmButton.text = _G[buttonName.."Text"]
    gemStatValConfirmButton.text:SetText("OK")
    gemStatValConfirmButton:Hide()


    -- On focus gain
    gemStatValEditBox:SetScript("OnEditFocusGained", function(self)
        gemStatValConfirmButton:Show()
        gemStatValEditBox:HighlightText()
    end)


    -- On focus lost
    gemStatValEditBox:SetScript("OnEditFocusLost", function(self)

        local currentSpecInfo = t.getVar("specs")[t.currentEditingSpec]

        gemStatValConfirmButton:Hide()
        gemStatValEditBox:HighlightText(0,0)
        --currentSpecInfo.sw[shortName] = gemStatValEditBox:GetNumber()
        --gemStatValEditBox:SetText(string.format("%.2f", t.svc.statWeights[shortName]))
        --gemStatValEditBox:SetText(string.format("%.2f", currentSpecInfo.sw[shortName]))
        currentSpecInfo["gemStatVal"] = gemStatValEditBox:GetNumber()
        gemStatValEditBox:SetText(string.format("%d", currentSpecInfo["gemStatVal"]))

    end)


    gemStatValEditBox:SetScript("OnEnterPressed",  gemStatValEditBox.ClearFocus)
    gemStatValEditBox:SetScript("OnTabPressed",  gemStatValEditBox.ClearFocus)
    gemStatValConfirmButton:SetScript("OnClick", function()
        gemStatValEditBox:ClearFocus()
    end)

















    --[[ RIGHT SIDE ]]--
    local rightFrame = CreateFrame("Frame", t.ADDON_NAME.."StatConfigFrameRight")
    rightFrame:SetWidth(frame:GetWidth() / 2 * uiScale)
    rightFrame:SetHeight(frame:GetHeight() * uiScale)
    rightFrame:SetPoint("TOPLEFT", leftFrame, "TOPRIGHT", 0,0)
    rightFrame:Show()

    local anchorTo = rightFrame


    
    t.options = t.options or {}
    t.options.swBoxes = {}
    local boxes = t.options.swBoxes

    local counter = 1

    local sw1  = {
        ITEM_MOD_INTELLECT_SHORT = "Intellect",
        ITEM_MOD_AGILITY_SHORT = "Agility",
        ITEM_MOD_STRENGTH_SHORT = "Strength",
    }
    local sw2 = {

        ITEM_MOD_HASTE_RATING_SHORT = "Haste",
        ITEM_MOD_MASTERY_RATING_SHORT = "Mastery",
        ITEM_MOD_CRIT_RATING_SHORT = "Crit",
        ITEM_MOD_VERSATILITY = "Versatility",
    }
    local sw3 = {
        ITEM_MOD_CR_AVOIDANCE_SHORT = "Avoidance",
        ITEM_MOD_CR_SPEED_SHORT = "Speed",
        ITEM_MOD_CR_LIFESTEAL_SHORT = "Leech"
    }
    local sw4 = {
        ITEM_MOD_STAMINA_SHORT = "Stamina",
        RESISTANCE0_NAME = "Armor"
    }


    counter, anchorTo = drawStatGroup("Primaries", sw1, boxes, counter, statweightFrame, anchorTo, "TOPLEFT")
    counter, anchorTo = drawStatGroup("Secondaries", sw2, boxes, counter, statweightFrame, anchorTo, "BOTTOMLEFT")
    counter, anchorTo = drawStatGroup("Tertiaries", sw3, boxes, counter, statweightFrame, anchorTo, "BOTTOMLEFT")
    counter, anchorTo = drawStatGroup("Extra", sw4, boxes, counter, statweightFrame, anchorTo, "BOTTOMLEFT")

    -- Set on enter scripts
    for n=1, #boxes do
        local s = boxes[n]

        local nextBoxFunction = function(self)
            self:ClearFocus()
            s.confirmButton:Hide()
            local nextIndex = IsShiftKeyDown() and n-1 or n+1
            if boxes[nextIndex] then
                boxes[nextIndex].editBox:SetFocus()
            end
        end
        s.editBox:SetScript("OnEnterPressed",  nextBoxFunction)
        s.editBox:SetScript("OnTabPressed",  nextBoxFunction)


        s.confirmButton:SetScript("OnClick", function()
            s.editBox:ClearFocus()
        end)
    end


    -- Import String title
    local importStringTitle = statweightFrame:CreateFontString(t.ADDON_NAME.."ImportStringTitle", "ARTWORK", "GameFontNormal")
    importStringTitle:SetPoint("TOPLEFT", anchorTo, "BOTTOMLEFT", 0,-10)
    importStringTitle:SetText("Import Pawn String")
    importStringTitle:SetJustifyH("LEFT")
    importStringTitle:Show()


    importEditBox = CreateFrame("EditBox", t.ADDON_NAME.."ImportEditBox", statweightFrame, "InputBoxTemplate")
    importEditBox:SetPoint("TOPLEFT", importStringTitle, "BOTTOMLEFT", 5, -5)
    importEditBox:SetText("")
    importEditBox:SetFont("Fonts\\FRIZQT__.TTF", 11, "OUTLINE")
    importEditBox:SetFrameStrata("FULLSCREEN")
    importEditBox:SetWidth(100)
    importEditBox:SetHeight(30)
    importEditBox:SetAutoFocus(false)
    importEditBox:Show()

    local buttonName = t.ADDON_NAME.."ImportConfirmButton"
    importConfirmButton = CreateFrame("Button", buttonName, statweightFrame, "UIPanelButtonTemplate")
    importConfirmButton:SetWidth(60)
    importConfirmButton:SetHeight(20)
    importConfirmButton:SetPoint("LEFT", importEditBox, "RIGHT", 5, 0)
    importConfirmButton.text = _G[buttonName.."Text"]
    importConfirmButton.text:SetText("Import")
    importConfirmButton:Show()




    -- On focus gain
    importEditBox:SetScript("OnEditFocusGained", function(self)
    end)


    -- On focus lost
    importEditBox:SetScript("OnEditFocusLost", function(self)

        local currentSpecInfo = t.getVar("specs")[t.currentEditingSpec]

        t.svc.defaultWeights = false

        local s = importEditBox:GetText()
        if s and (s:gsub("%s+", "") ~= "") then
            local sw = t.parsePawnString(s)

            if sw then
                for stat,val in pairs(sw) do
                    local shortStat = t.fromPawnStatName[stat]
                    if shortStat then
                        currentSpecInfo.sw[shortStat] = val
                    end
                end


                -- Refresh the panel so that editboxes get updated
                frame:Hide()
                frame:Show()
            else
                print(t.MSG_PREFIX.."Could not parse Pawn string succesfully.")
            end
        end

        importEditBox:SetText("")
    end)

    importEditBox:SetScript("OnEnterPressed", function() 
        importEditBox:ClearFocus()

    end)

    importConfirmButton:SetScript("OnClick", function()
        importEditBox:ClearFocus()
    end)






    local Refresh;
    function Refresh()
        if not frame:IsVisible() then return end

        currentSpecInfo = specInfo[t.currentEditingSpec]

        -- Update checkboxes
        enableCheckbox:SetChecked(currentSpecInfo["enable"])
        msCheckbox:SetChecked(currentSpecInfo["ms"])

        -- Update gem dropdown
        local currentGemStat = currentSpecInfo["gemStat"]
        local selectedId = 1
        local i = 1
        for shortName, fullName in pairs(t.reverseStm) do
            i = i + 1
            if fullName == currentGemStat then
                selectedId = i
            end
        end

        UIDropDownMenu_Initialize(_G[t.ADDON_NAME.."GemStatDropdown"], initGemStat)
        UIDropDownMenu_SetSelectedID(_G[t.ADDON_NAME.."GemStatDropdown"], selectedId)

        -- Update gem stat value

        gemStatValEditBox:SetText(string.format("%d", currentSpecInfo["gemStatVal"]))


        -- Update equipment set dropdwo
        --[[
        local selectedId = nil
        for i=1, GetNumEquipmentSets() do
            if GetEquipmentSetInfo(i) == currentSpecInfo["equipmentSet"] then
                selectedId = i+1
            end
        end
        if not selectedId then
            selectedId = 1
        end
        ]]
        local set = currentSpecInfo["equipmentSet"]
        if set == -1 then selectedId = 1 else selectedId = set+2 end



        UIDropDownMenu_Initialize(_G[t.ADDON_NAME.."EquipmentSetDropdown"], initEquipmentSets)
        UIDropDownMenu_SetSelectedID(_G[t.ADDON_NAME.."EquipmentSetDropdown"], selectedId)


        -- Hide or show depending on Enable box
        if currentSpecInfo["enable"] then
            msCheckbox:Show()
            for i=1, #boxes do
                boxes[i].statBox:Show()
                boxes[i].statText:Show()
                boxes[i].editBox:Show()

                local shortName = boxes[i].shortName
                local statWeightValue = string.format("%.2f", (currentSpecInfo.sw[shortName] or 0))
                boxes[i].editBox:SetText(statWeightValue)
            end
            _G[t.ADDON_NAME.."PrimariesTitle"]:Show()
            _G[t.ADDON_NAME.."SecondariesTitle"]:Show()
            _G[t.ADDON_NAME.."ExtraTitle"]:Show()
            _G[t.ADDON_NAME.."EquipmentSetDropdown"]:Show()
            _G[t.ADDON_NAME.."EquipmentSetDropdownTitle"]:Show()
            _G[t.ADDON_NAME.."GemStatDropdown"]:Show()
            _G[t.ADDON_NAME.."GemStatValEditBox"]:Show()
            _G[t.ADDON_NAME.."GemStatDropdownTitle"]:Show()
        else
            msCheckbox:Hide()

            for i=1, #boxes do
                boxes[i].statBox:Hide()
                boxes[i].statText:Hide()
                boxes[i].confirmButton:Hide()
                boxes[i].editBox:Hide()
            end
            _G[t.ADDON_NAME.."PrimariesTitle"]:Hide()
            _G[t.ADDON_NAME.."SecondariesTitle"]:Hide()
            _G[t.ADDON_NAME.."TertiariesTitle"]:Hide()
            _G[t.ADDON_NAME.."ExtraTitle"]:Hide()
            _G[t.ADDON_NAME.."EquipmentSetDropdown"]:Hide()
            _G[t.ADDON_NAME.."EquipmentSetDropdownTitle"]:Hide()
            _G[t.ADDON_NAME.."GemStatDropdown"]:Hide()
            _G[t.ADDON_NAME.."GemStatValEditBox"]:Hide()
            _G[t.ADDON_NAME.."GemStatDropdownTitle"]:Hide()

        end
    end

    frame:SetScript("OnShow", Refresh) 
    Refresh()
end)


t.parsePawnString = function(s)

    local weights = {}
    local pattern = "%(?.*:.*:(.*)%)?"
    local values = s:match(pattern)
    if values then
        for str in values:gmatch("([^),]*)") do
            local stat, weight = str:match("([^=]*)=([^=]*)")
            if stat and weight then
                stat = stat:gsub("%s+", "")
                weights[stat] = tonumber(weight)
            end
        end


        return weights
    else
        return nil
    end
end


InterfaceOptions_AddCategory(statweightFrame)

